# Regression data set
#####################################

rm(list=ls())
set.seed(1280)
x<-runif(50)
y<-3+2*x+rnorm(50, mean=0, sd=2)

dat<-data.frame(y, x)


# Bayesian regression in R+WinBUGS
# or R+OpenBUGS
#####################################


library(arm)
library(BRugs)

N<-length(x)

data<- list("N", "y", "x")
inits <- function() {list (alpha = 0, beta = 0, tau.y = 1)}
parameters<- c("alpha", "beta", "tau.y")
regression.sim<-bugs(data, inits, parameters.to.save=parameters, model.file="example1.bug", n.chains=4, n.iter=5000, n.burnin=1000, n.thin=1, debug=F, program="WinBUGS")

geweke.diag(regression.sim)
heidel.diag(as.mcmc(regression.sim))
raftery.diag(as.mcmc(regression.sim))

print(regression.sim)

library(mcmcplots)
mcmcplot(regression.sim, dir=getwd())

attach.bugs(regression.sim)

plot(density(alpha))
abline(v=3, lty=2)
quantile(alpha, probs=c(0.025, 0.975))

plot(density(beta))
abline(v=2, lty=2)
quantile(beta, probs=c(0.025, 0.975))

plot(density(tau.y))
plot(density(sqrt(1/tau.y)))
abline(v=2, lty=2)
quantile(sqrt(1/tau.y), probs=c(0.025, 0.975))

detach.bugs()












regression.sim<-bugs(data, inits, parameters, "example1.bug", n.chains=4, n.iter=5000, n.burnin=1000, n.thin=1, debug=T, program="OpenBUGS")

print(regression.sim)
mcmcplot(regression.sim, dir=getwd())


geweke.diag(regression.sim)
heidel.diag(regression.sim)
raftery.diag(regression.sim)



attach.bugs(regression.sim)

plot(alpha, type="l")
plot(beta, type="l")
plot(tau.y, type="l")

plot(density(alpha))
plot(density(beta))
plot(density(tau.y))

plot(alpha~beta)

detach.bugs()









# Bayesian regression in MCMCPack
#####################################

library(MCMCpack)

regression.sim<-MCMCregress(y~x, data = dat, burnin = 1000, mcmc = 5000, thin = 1, verbose = 0, seed = 04928104, beta.start = NA, b0 = c(0,0), B0 = c(0.001,0.001), c0 = 1, d0 = 3)

plot(regression.sim)
summary(regression.sim)








# binary DV (logistic) data set
#####################################

rm(list=ls())
set.seed(348902340)

x<-runif(50)
xb<-(-2+2*x)
ystar<-rlogis(50, location=xb, scale=1)
y<-as.numeric(ystar>0)

dat<-data.frame(y, x)


# Bayesian logit in R+WinBUGS
# or R+OpenBUGS
#####################################


library(arm)
library(BRugs)

N<-length(x)

data<- list("N", "y", "x")
inits <- function() {list (alpha = 0, beta = 0)}
parameters<- c("alpha", "beta")
logit.sim<-bugs(data, inits, parameters, "example2.bug", n.chains=4, n.iter=25000, n.burnin=500, n.thin=1, debug=F, program="WinBUGS")

geweke.diag(logit.sim)
heidel.diag(as.mcmc(logit.sim))
raftery.diag(as.mcmc(logit.sim))

print(logit.sim)
mcmcplot(logit.sim, dir=getwd())

attach.bugs(logit.sim)

plot(density(alpha))
plot(density(beta))

detach.bugs()










# Regression with Unit Heterogeneity data set
#####################################

rm(list=ls())
set.seed(129382013)

unit<-c()
x<-c()
y<-c()

for(i in 1:20){
  
  unit.t<-rep(i, 50)
  unit.effect<-rnorm(1, mean=0, sd=3)
  x.t<-runif(50)
  y.t<-3+2*x.t+rnorm(50, mean=0, sd=2)+unit.effect
  
  unit<-c(unit, unit.t)
  x<-c(x, x.t)
  y<-c(y, y.t)
  
}

dat<-data.frame(y, x, unit)



# Bayesian RE regression in R+WinBUGS
# or R+OpenBUGS
#####################################


library(arm)
library(BRugs)

N<-length(x)
M<-length(unique(unit))

data<- list("N", "M", "unit", "y", "x")
inits <- function() {list (alpha = 0, beta = 0, tau.y = 1, tau.re = 0.5, ranef.v = rep(0, 20))}
parameters<- c("alpha", "beta", "tau.y", "tau.re")
re.sim<-bugs(data, inits, parameters, "example3.bug", n.chains=1, n.iter=10000, n.burnin=1000, n.thin=1, debug=F, program="openbugs")

geweke.diag(re.sim)
heidel.diag(as.mcmc(re.sim))
raftery.diag(as.mcmc(re.sim))

library(mcmcplots)
mcmcplot(re.sim, dir=getwd())

attach.bugs(re.sim)

plot(density(alpha))
plot(density(beta))
plot(density(sqrt(1/tau.re)))

detach.bugs()














# Cauchy data
#####################################
rm(list=ls())
set.seed(123821)

x<-runif(50)
y<-rcauchy(50, location=2*x+1, scale=0.5)
dat<-data.frame(x,y)


# Cauchy likelihood model
#####################################


cauchy.like<-function(b, y, x){

  bb<-b[1:(length(b)-1)]
  gamma<-b[length(b)]
  out<-1/(pi*gamma*(1+((y-x%*%bb)/gamma)^2))
  out<-pmax(out, 1e-10)
  return(sum(log(out)))

}

prior<-function(b){
  
  p.1<-dnorm(b[1], mean=0, sd=3)
  p.2<-dnorm(b[2], mean=0, sd=3)
  
  if(b[3]>0){
    p.3<-dinvgamma(b[3], shape=.001, scale=3)
  }else{p.3<-1e-10}
  
  p.1<-max(p.1, 1e-10)
  p.2<-max(p.2, 1e-10)
  p.3<-max(p.3, 1e-10)
  
  return(log(p.1)+log(p.2)+log(p.3))
  
}

cauchy.post<-function(b,y,x){
  
  out<-cauchy.like(b,y,x)+prior(b)
  return(out)
  
}

cauchy.sim<-MCMCmetrop1R(cauchy.post, theta.init=c(1,1,1), y=y, x=cbind(1,x), burnin=500, mcmc=20000)

plot(cauchy.sim)
summary(cauchy.sim)

geweke.diag(cauchy.sim)
heidel.diag(cauchy.sim)
raftery.diag(cauchy.sim)
